#[repr(C)]pub struct VMCallerCheckedAnyfunc {
pub func_ptr: *const VMFunctionBody,
pub type_index: VMSharedSignatureIndex,
pub vmctx: *mut VMContext,
}Expand description
The VM caller-checked “anyfunc” record, for caller-side signature checking. It consists of the actual function pointer and a signature id to be checked by the caller.
Fields
func_ptr: *const VMFunctionBodyFunction body.
type_index: VMSharedSignatureIndexFunction signature id.
vmctx: *mut VMContextFunction VMContext.