#[repr(C)]pub struct VMMemoryDefinition {
pub base: *mut u8,
pub current_length: usize,
}Expand description
The fields compiled code needs to access to utilize a WebAssembly linear memory defined within the instance, namely the start address and the size in bytes.
Fields
base: *mut u8The start address.
current_length: usizeThe current logical size of this linear memory in bytes.